<?php

namespace FM\Service;

use FM\Factory\Fm00040000ModeloAuditoriaFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FindService;
use FM\Service\FileServerService;
use FM\Service\FuncionariosHierarqService;

class Fm00040000ModeloAuditoriaService
{

    protected $ModeloAuditoriaFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;
    protected $FindService;
    protected $FileServerService;
    protected $fileServerUrl;
    protected $FuncionariosHierarqService;

    public function __construct(Fm00040000ModeloAuditoriaFactory $ModeloAuditoriaFactory, Environment $Environment, NewCode $newCode, EntityManager $entityManager, FindService $FindService, FileServerService $FileServerService, $fileServerUrl, FuncionariosHierarqService $FuncionariosHierarqService)
    {
        $this->connection = $entityManager->getConnection();
        $this->ModeloAuditoriaFactory = $ModeloAuditoriaFactory;
        $this->Environment = $Environment;
        $this->newCode = $newCode;
        $this->FindService = $FindService;
        $this->FileServerService = $FileServerService;
        $this->fileServerUrl = $fileServerUrl;
        $this->FuncionariosHierarqService = $FuncionariosHierarqService;
    }

    public function getTarefasAuditoria()
    {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        
        $params = array(
            "CDOPERADOR" => $cdoperador
        );

        return $this->ModeloAuditoriaFactory->getTarefasAuditoria($listaOperadores, $params);
    }

    public function getCriteriosAuditoria()
    {
        return $this->ModeloAuditoriaFactory->getCriteriosAuditoria();
    }

    public function getImagensAuditoria($parametros)
    {
        return $this->ModeloAuditoriaFactory->getImagensAuditoria($parametros);
    }


    public function inserirTarefaAuditoria($parametros)
    {

        $dados = $this->montarDadosTarefa($parametros);
        $parametros = $this->validaDiasUteis($dados);

        if ($parametros['RECORRENCIA'] !== 'SR') {
            $this->insertTarefa($parametros);
            do {
                if ($parametros['TERMINARECO'] == 'N') {
                    $parametros = $this->calcularRecorrenciaPeriodo($parametros);
                    $this->insertTarefa($parametros);
                }
            } while ($parametros['TERMINARECO'] == 'N');
        } else {
            $this->insertTarefa($parametros);
        }
    }
    public function excluirAuditoria($parametros)
    {
        $transacaoAtiva = false;
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();
    
            if (!isset($parametros['AUDITORIA']) || !is_array($parametros['AUDITORIA'])) {
                $this->connection->rollback();
                return [
                    'status' => 'erro',
                    'mensagem' => 'Parâmetros de auditoria inválidos.',
                ];
            }
    
            $auditorias = implode(',', array_map('intval', $parametros['AUDITORIA']));
            $situacoes = $this->ModeloAuditoriaFactory->validaSituacao($auditorias);
            foreach ($situacoes as $situacao) {
                if ($situacao['IDSITUACAOAUDIT'] != 1) {
                    $this->connection->rollback();
                    return [
                        'status' => 'erro',
                        'mensagem' => 'Somente auditorias com situação válida podem ser excluídas.',
                    ];
                }
            }
    
            foreach ($parametros['AUDITORIA'] as $auditoria) {
                $this->ModeloAuditoriaFactory->deleteAuditoria([
                    'NRAUDITORIA' => $auditoria,
                    "CDOPERULTATU" => $cdOperador,
                    "NRORGULTATU" => $NRORG,
                ]);
            }
    
            $this->connection->commit();
            $transacaoAtiva = false;
    
            return [
                'status' => 'sucesso',
                'mensagem' => 'Auditoria(s) excluída(s) com sucesso.',
            ];
        } catch (\Exception $e) {
            if ($transacaoAtiva) {
                $this->connection->rollback();
            }
            throw $e;
        }
    }    
    public function insertTarefa($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            foreach ($parametros["EXECUTOR"] as $dados) {
                if ($dados["GERATAREFA"] == 'S') {
                    $dataInicial = new \DateTime($dados['DATAAUDIT']);
                    $nrAuditoria = $this->newCode->getCodeByTableName("FCM_AUDITORIA", $NRORG, 1, 10, true);

                    $this->ModeloAuditoriaFactory->insertTarefaAuditoria(array(
                        "NRAUDITORIA" => $nrAuditoria,
                        "NMAUDITORIA" => $parametros['NMAUDITORIA'],
                        "NRLOCALCLIENTE" => $dados['NRLOCALCLIENTE'],
                        "NRMODELOAUDIT" => $parametros['NRMODELOAUDIT'],
                        "NRSEQCONTRATO" => isset($dados['NRSEQCONTRATO']) ? $dados['NRSEQCONTRATO'] : null,
                        "CDOPERADOR" => isset($dados['CDOPERADOR'], ) ? $dados['CDOPERADOR'] : null,
                        "DTHORAAUDIT" => $dataInicial->format('d/m/Y'),
                        "NRORG" => $NRORG,
                        "CDOPERINCLUSAO" => $cdOperador,
                        "NRORGINCLUSAO" => $NRORG,
                        "IDOBRIGAOBSERVACAO" => $parametros['OBSERVACOES'],
                        "IDTIPORECORRENCIA" => $parametros['RECORRENCIA'],
                        "IDTIPOMARCACAO" => $dados['IDTIPOMARCACAO'],
                    ));

                    $topicos = $this->ModeloAuditoriaFactory->getTopicosModelo(array(
                        "NRMODELOAUDIT" => $parametros['NRMODELOAUDIT'],
                    ));

                    foreach ($topicos as $topico) {
                        $nrResultado = $this->newCode->getCodeByTableName("FCM_RESULTADOAUDIT", $NRORG, 1, 10, true);
                        $this->ModeloAuditoriaFactory->inserirTopicosExecucao(array(
                            "NRRESULTADOAUDIT" => $nrResultado,
                            "NRAUDITORIA" => $nrAuditoria,
                            "NRMODELOAUDIT" => $parametros['NRMODELOAUDIT'],
                            "NRTOPICOAUDIT" => $topico['NRTOPICOAUDIT'],
                            "NRCATEGORIAAUDIT" => $topico['NRCATEGORIAAUDIT'],
                            "NRORG" => $NRORG,
                            "CDOPERINCLUSAO" => $cdOperador,
                            "NRORGINCLUSAO" => $NRORG,
                        ));

                        $criterios = $this->ModeloAuditoriaFactory->getCriteriosModelo(array(
                            "NRCATEGORIAAUDIT" => $topico['NRCATEGORIAAUDIT'],
                        ));

                        foreach ($criterios as $criterio) {
                            $this->ModeloAuditoriaFactory->inserirCriteriosExecucao(array(
                                "NRRELACRESULTCRIT" => $this->newCode->getCodeByTableName("FCM_RELACRESULTCRIT", $NRORG, 1, 10, true),
                                "NRRESULTADOAUDIT" => $nrResultado,
                                "NRCRITERIOAUDIT" => $criterio['NRCRITERIOAUDIT'],
                                "NRTOPICOAUDIT" => $topico['NRTOPICOAUDIT'],
                                "NRORG" => $NRORG,
                                "CDOPERINCLUSAO" => $cdOperador,
                                "NRORGINCLUSAO" => $NRORG,
                            ));
                        }
                    }
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    // funções da tab de Modelo
    public function excluirModelo($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $modelo = [
                "NRMODELOAUDIT" => $parametros['MODELO'],
            ];

            $valida = $this->ModeloAuditoriaFactory->validaModeloAuditoria($modelo);

            if (empty($valida)) {
                $categorias = $this->ModeloAuditoriaFactory->getCategoriaPorModelo($modelo);

                $categoriasExcluir = array_map(
                    fn($cat) => intval($cat['NRCATEGORIAAUDIT']),
                    $categorias
                );
                if (!empty($categoriasExcluir)) {
                    $this->ModeloAuditoriaFactory->deleteRelacoesModelo(
                        $this->FindService->formatarArray($categoriasExcluir),
                        $parametros['MODELO']
                    );
                }
                $this->ModeloAuditoriaFactory->deleteModelo($modelo);

                $this->connection->commit();

                return [
                    'status' => 'sucesso',
                    'mensagem' => 'Modelo excluído com sucesso.',
                ];
            }

            $relacionados = array_map(
                fn($rel) => [
                    'NMAUDITORIA' => $rel['NMAUDITORIA'] ?? 'Nome da auditoria não disponível',
                    'NRAUDITORIA' => $rel['NRAUDITORIA'] ?? 'Número da auditoria não disponível',
                    'IDSITUACAOAUDIT' => $rel['IDSITUACAOAUDIT'] ?? 'Situação não disponível',
                    'DTHORAAUDIT' => $rel['DTHORAAUDIT'] ?? 'Data/Hora não disponível',
                ],
                $valida
            );

            return [
                'status' => 'erro',
                'mensagem' => 'Não é possível excluir. Existem elementos relacionados a este modelo.',
                'relacionados' => $relacionados,
            ];
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inativarModelo($parametros)
    {
        $transacaoAtiva = false;
        $NRORG = $this->Environment->getNrOrg();
        $cdOperador = $this->Environment->getCdOperador();

        try {
            $this->connection->beginTransaction();
            $transacaoAtiva = true;

            if ($parametros['ATIVAR']) {
                $param = [
                    "NRMODELOAUDIT" => $parametros['NRMODELOAUDIT'],
                    "CDOPERULTATU" => $cdOperador,
                    "NRORGULTATU" => $NRORG,
                ];

                $this->ModeloAuditoriaFactory->ativarModelo($param);
            } else {
                $modelo = [
                    "NRMODELOAUDIT" => $parametros['NRMODELOAUDIT'],
                ];

                $valida = $this->ModeloAuditoriaFactory->validaModeloAuditoriaInatividade($modelo);
                if (!empty($valida)) {
                    $relacionadosNaoConcluidos = array_filter($valida, fn($rel) => $rel['IDSITUACAOAUDIT'] != 3);

                    if (!empty($relacionadosNaoConcluidos)) {
                        $relacionados = array_map(
                            fn($rel) => [
                                'NMAUDITORIA' => $rel['NMAUDITORIA'] ?? 'Nome da auditoria não disponível',
                                'NRAUDITORIA' => $rel['NRAUDITORIA'] ?? 'Número da auditoria não disponível',
                                'IDSITUACAOAUDIT' => $rel['IDSITUACAOAUDIT'] ?? 'Situação não disponível',
                                'DTHORAAUDIT' => $rel['DTHORAAUDIT'] ?? 'Data/Hora não disponível',
                            ],
                            $valida
                        );

                        $this->connection->rollback();
                        $transacaoAtiva = false;

                        return [
                            'status' => 'erro',
                            'mensagem' => 'Não é possível inativar. Existem elementos relacionados a este modelo com status diferente de "Concluído".',
                            'relacionados' => $relacionados,
                        ];
                    }
                }

                $modelo = [
                    "NRMODELOAUDIT" => $parametros['NRMODELOAUDIT'],
                    "CDOPERULTATU" => $cdOperador,
                    "NRORGULTATU" => $NRORG,
                ];

                $this->ModeloAuditoriaFactory->inativaModelo($modelo);
            }

            $this->connection->commit();
            $transacaoAtiva = false;

            return [
                'status' => 'sucesso',
                'mensagem' => 'Modelo inativado com sucesso.',
            ];
        } catch (\Exception $e) {
            if ($transacaoAtiva) {
                $this->connection->rollback();
            }
            throw $e;
        }
    }

    public function excluirDados($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $arrayTopicos = [];
            $arrayCategorias = [];

            $modelo = [
                "NRMODELOAUDIT" => $parametros['MODELO'],
            ];

            $validaModelo = $this->ModeloAuditoriaFactory->validaModeloAuditoria($modelo);

            if (!empty($validaModelo)) {
                $relacionados = array_map(
                    fn($rel) => [
                        'NMAUDITORIA' => $rel['NMAUDITORIA'] ?? 'Nome da auditoria não disponível',
                        'NRAUDITORIA' => $rel['NRAUDITORIA'] ?? 'Número da auditoria não disponível',
                        'IDSITUACAOAUDIT' => $rel['IDSITUACAOAUDIT'] ?? 'Situação não disponível',
                        'DTHORAAUDIT' => $rel['DTHORAAUDIT'] ?? 'Data/Hora não disponível',
                    ],
                    $validaModelo
                );
                return [
                    'status' => 'erro',
                    'mensagem' => 'Não é possível excluir. Existem elementos relacionados a este modelo.',
                    'relacionados' => $relacionados,
                ];
            }

            $valida = $this->ModeloAuditoriaFactory->validaModelo([
                "NRMODELOAUDIT" => $parametros['MODELO'],
            ]);

            if (intval($valida['CONT'] ?? 0) === 0) {
                foreach ($parametros['DADOS'] as $codigo) {
                    if (substr($codigo, 0, 1) === 'C') {
                        array_push($arrayCategorias, substr($codigo, 1));
                    } else {
                        array_push($arrayTopicos, substr($codigo, 1));
                    }
                }

                if (count($arrayTopicos) > 0) {
                    $this->ModeloAuditoriaFactory->deleteRelacaoCatTopicos(
                        $this->FindService->formatarArray($arrayTopicos),
                        $parametros['MODELO']
                    );
                }

                if (count($arrayCategorias) > 0) {
                    $this->ModeloAuditoriaFactory->deleteRelacaoCatCriterios(
                        $this->FindService->formatarArray($arrayCategorias),
                        $parametros['MODELO']
                    );
                    $this->ModeloAuditoriaFactory->deleteRelacaoCatModelo(
                        $this->FindService->formatarArray($arrayCategorias),
                        $parametros['MODELO']
                    );
                }
            }

            $this->connection->commit();
            return $valida['CONT'] ?? 0;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function montarDadosTarefa($parametros)
    {
        $intervaloDias = $this->geraItervalo($parametros['DATERANGE'][0], (isset($parametros['DATERANGE'][1]) ? $parametros['DATERANGE'][1] : $parametros['DATERANGE'][0]));



        foreach ($parametros['EXECUTOR'] as $key => $dados) {
            foreach ($intervaloDias as $dia) {
                $dados['DATAAUDIT'] = $dia;
                unset($parametros['EXECUTOR'][$key]);
                array_push($parametros['EXECUTOR'], $dados);
            }
        }
        return $parametros;
    }

    public function geraItervalo($dataInicio, $dataFim)
    {
        // Definir um intervalo de 1 dia
        $fim = new \DateTime($dataFim);
        $intervalo = new \DateInterval('P1D');
        $periodo = new \DatePeriod(new \DateTime($dataInicio), $intervalo, $fim->modify('+1 day'));

        $datas = array();
        foreach ($periodo as $data) {
            array_push($datas, $data->format('m/d/Y'));
        }
        return $datas;
    }

    public function validaDiasUteis($parametros)
    {
        foreach ($parametros['EXECUTOR'] as $key => $exec) {
            $dataInicio = new \DateTime($exec['DATAAUDIT']);
            $exec['DATAINICIALBKP'] = $dataInicio->format('m/d/Y H:i');

            $diaDaSemana = $dataInicio->format('N');
            $dt = $dataInicio->format('m/d/Y');

            // Verificar se a data é um dia útil
            if ($parametros['DIASEMANA'] == '2') {
                if ($diaDaSemana == 6) { // sábado
                    $dataInicio->add(new \DateInterval('P2D')); // soma dois dias para retornar à sexta-feira
                } elseif ($diaDaSemana == 7) { // domingo
                    $dataInicio->add(new \DateInterval('P1D')); // soma um dias para retornar ao último dia útil anterior, que é sexta-feira
                }
                $exec['GERATAREFA'] = 'S';
            } elseif ($parametros['DIASEMANA'] == '3') {
                $exec['GERATAREFA'] = ($diaDaSemana == 6 || $diaDaSemana == 7) ? 'N' : 'S';
                $parametros['EXECUTOR'][$key] = $exec;
            } elseif ($parametros['DIASEMANA'] == '1') {
                $exec['GERATAREFA'] = 'S';
                $parametros['EXECUTOR'][$key] = $exec;
            }

            $dataTermino = (isset($parametros['DTTERMINO'])) ? $parametros['DTTERMINO'] : ((isset($parametros['DATERANGE'][1])) ? $parametros['DATERANGE'][1] : $parametros['DATERANGE'][0]);
            if (new \DateTime($dt) <= new \DateTime($dataTermino)) {
                $exec['DATAAUDIT'] = $dataInicio->format('m/d/Y');
                $parametros['EXECUTOR'][$key] = $exec;
                $parametros['TERMINARECO'] = 'N';
            } else {
                $parametros['TERMINARECO'] = 'S';
                unset($parametros['EXECUTOR'][$key]);
            }
        }

        return $parametros;
    }

    public function calcularRecorrenciaPeriodo($parametros)
    {
        // Definir a frequência da tarefa
        switch ($parametros['RECORRENCIA']) {
            case 'Q':
                $frequencia = new \DateInterval('P14D');
                break;
            case 'M':
                $frequencia = new \DateInterval('P1M');
                break;
            case 'S':
                $frequencia = new \DateInterval('P179D');
                break;
            case 'QP':
                $frequencia = new \DateInterval('P14D');
                break;
            case 'MP':
                $frequencia = new \DateInterval('P1M');
                break;
            case 'SP':
                $frequencia = new \DateInterval('P179D');
                break;
            case 'P':
                $frequencia = new \DateInterval('P' . $parametros['QTDDIAS'] . 'D');
                break;
        }

        // Loop para adicionar as datas das próximas tarefas
        foreach ($parametros['EXECUTOR'] as $key => $executor) {
            $dataInicio = new \DateTime($executor['DATAINICIALBKP']);

            $dataInicio = $dataInicio->add($frequencia);
            $executor['DATAAUDIT'] = $dataInicio->format('m/d/Y');
            $parametros['EXECUTOR'][$key] = $executor;
        }

        return $this->validaDiasUteis($parametros);
    }

    // Funções Tab Modelo
    public function getEstruturaModelo($parametros)
    {
        return $this->ModeloAuditoriaFactory->getEstruturaModelo(array(
            "NRMODELOAUDIT" => $parametros['NRMODELOAUDIT']
        ));
    }

    public function salvarModelo($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            $validar = $this->ModeloAuditoriaFactory->validarCategoria(array(
                "NRCATEGORIAAUDIT" => $parametros['NRCATEGORIAAUDIT'],
                "NRMODELOAUDIT" => $parametros['NRMODELOAUDIT'],
            ));

            if (intval($validar['CONT']) === 0) {
                $this->ModeloAuditoriaFactory->insertRelacaoModCategoria(array(
                    "NRRELACMODCATEG" => $this->newCode->getCodeByTableName("FCM_RELACMODCATEG", $NRORG, 1, 10, true),
                    "NRMODELOAUDIT" => $parametros['NRMODELOAUDIT'],
                    "NRCATEGORIAAUDIT" => $parametros['NRCATEGORIAAUDIT'],
                    "NRORG" => $NRORG,
                    "CDOPERINCLUSAO" => $cdOperador,
                    "NRORGINCLUSAO" => $NRORG
                ));


                foreach ($parametros['NRCRITERIOAUDIT'] as $criterios) {
                    $this->ModeloAuditoriaFactory->insertRelacaoCategoriaCriterio(array(
                        "NRRELACCATEGCRIT" => $this->newCode->getCodeByTableName("FCM_RELACCATEGCRIT", $NRORG, 1, 10, true),
                        "NRCATEGORIAAUDIT" => $parametros['NRCATEGORIAAUDIT'],
                        "NRMODELOAUDIT" => $parametros['NRMODELOAUDIT'],
                        "NRCRITERIOAUDIT" => $criterios,
                        "NRORG" => $NRORG,
                        "CDOPERINCLUSAO" => $cdOperador,
                        "NRORGINCLUSAO" => $NRORG
                    ));
                }
            }

            foreach ($parametros['NRTOPICOAUDIT'] as $topicos) {
                $validaTopico = $this->ModeloAuditoriaFactory->validarTopicos(array(
                    "NRCATEGORIAAUDIT" => $parametros['NRCATEGORIAAUDIT'],
                    "NRTOPICOAUDIT" => $topicos,
                    "NRMODELOAUDIT" => $parametros['NRMODELOAUDIT'],
                ));

                if (intval($validaTopico['CONT']) === 0) {
                    $this->ModeloAuditoriaFactory->insertRelacaoCategoriaTopico(array(
                        "NRRELACCATTOPIC" => $this->newCode->getCodeByTableName("FCM_RELACCATTOPIC", $NRORG, 1, 10, true),
                        "NRCATEGORIAAUDIT" => $parametros['NRCATEGORIAAUDIT'],
                        "NRMODELOAUDIT" => $parametros['NRMODELOAUDIT'],
                        "NRTOPICOAUDIT" => $topicos,
                        "NRORG" => $NRORG,
                        "CDOPERINCLUSAO" => $cdOperador,
                        "NRORGINCLUSAO" => $NRORG
                    ));
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function salvarModeloAuditoria($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            $this->ModeloAuditoriaFactory->insertModeloAuditoria(array(
                "NRMODELOAUDIT" => $this->newCode->getCodeByTableName("FCM_MODELOAUDIT", $NRORG, 1, 10, true),
                "NMMODELOAUDIT" => $parametros['NMMODELOAUDIT'],
                "NRORG" => $NRORG,
                "CDOPERINCLUSAO" => $cdOperador,
                "NRORGINCLUSAO" => $NRORG
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getModeloAuditoria($parametros)
    {
        $filtro = '';
        if (!empty($parametros['IDATIVO'])) {
            $filtro = " WHERE FCM_MODELOAUDIT.IDATIVO = " . "'".$parametros['IDATIVO']."'";
        }

        return $this->ModeloAuditoriaFactory->getModeloAuditoria($filtro);
    }

    // Funções da tab de Categoria

    public function salvarCategoriaAuditoria($parametros)
    {
        $this->connection->beginTransaction();
    
        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();
    
            if ($parametros["NRCATEGORIAAUDIT"] == 0) {
                $param = array(
                    "NRCATEGORIAAUDIT" => $this->newCode->getCodeByTableName("FCM_CATEGORIAAUDIT", $NRORG, 1, 10, true),
                    "NMCATEGORIAAUDIT" => $parametros["NMCATEGORIAAUDIT"],
                    "IDATIVO" => $parametros["IDATIVO"],
                    "NRORG" => $NRORG,
                    "NRORGINCLUSAO" => $NRORG,
                    "CDOPERINCLUSAO" => $cdOperador
                );
    
                $this->ModeloAuditoriaFactory->insertCategoriaAuditoria($param);
            } else if ($parametros["IDATIVO"] === 'S') {
                $NRCATEGORIAAUDIT = is_array($parametros["NRCATEGORIAAUDIT"]) 
                    ? $parametros["NRCATEGORIAAUDIT"][0] 
                    : $parametros["NRCATEGORIAAUDIT"];
    
                $param = array(
                    "NRCATEGORIAAUDIT" => $NRCATEGORIAAUDIT,
                    "NMCATEGORIAAUDIT" => $parametros["NMCATEGORIAAUDIT"],
                    "IDATIVO" => $parametros["IDATIVO"],
                    "CDOPERULTATU" => $cdOperador,
                    "NRORGULTATU" => $NRORG
                );
                $this->ModeloAuditoriaFactory->updateCategoriaAuditoria($param);
            } else {
                if (!is_array($parametros['NRCATEGORIAAUDIT'])) {
                    $parametros['NRCATEGORIAAUDIT'] = [$parametros['NRCATEGORIAAUDIT']];
                }
    
                $categorias = implode(',', array_map('intval', $parametros['NRCATEGORIAAUDIT']));
                $tarefasAssociadas = $this->ModeloAuditoriaFactory->getRelacaoCategoriasInativa($categorias);
    
                if (is_array($tarefasAssociadas) && !empty($tarefasAssociadas)) {
                    return [
                        'status' => 'erro',
                        'mensagem' => 'Não é possível excluir. Existem categorias relacionadas a outros elementos.',
                        'relacionados' => $tarefasAssociadas,
                    ];
                } else if (is_int($tarefasAssociadas) && $tarefasAssociadas > 0) {
                    return $tarefasAssociadas;
                } else {
                    $NRCATEGORIAAUDIT = is_array($parametros["NRCATEGORIAAUDIT"]) 
                        ? $parametros["NRCATEGORIAAUDIT"][0] 
                        : $parametros["NRCATEGORIAAUDIT"];
    
                    $param = array(
                        "NRCATEGORIAAUDIT" => $NRCATEGORIAAUDIT,
                        "NMCATEGORIAAUDIT" => $parametros["NMCATEGORIAAUDIT"],
                        "IDATIVO" => $parametros["IDATIVO"],
                        "CDOPERULTATU" => $cdOperador,
                        "NRORGULTATU" => $NRORG
                    );
                    $this->ModeloAuditoriaFactory->updateCategoriaAuditoria($param);
                }
            }
    
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }    
      
    public function getCategoriaAuditoria()
    {
        return $this->ModeloAuditoriaFactory->getCategoriaAuditoria();
    }

    public function excluirCategoria($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $mensagem = ['status' => 'ok', 'excluidos' => 0, 'nao_excluidos' => []];

            $categorias = $this->ModeloAuditoriaFactory->getRelacaoCategorias(
                $this->FindService->formatarArray($parametros['CATEGORIAS'])
            );

            $categoriasExcluir = [];
            $categoriasRelacionadasMap = [];

            foreach ($parametros['CATEGORIAS'] as $categoria) {
                $dataParam = ["NRCATEGORIAAUDIT" => $categoria];
                $relacionamentos = $this->ModeloAuditoriaFactory->verificaRelacionamentoCategorias($dataParam);

                if (empty($relacionamentos)) {
                    $categoriasExcluir[] = intval($categoria);
                } else {
                    foreach ($relacionamentos as $rel) {
                        $key = $rel['NRMODELOAUDIT'] . '-' . $rel['NMMODELOAUDIT'] . '-';

                        if (!isset($categoriasRelacionadasMap[$key])) {
                            $categoriasRelacionadasMap[$key] = [
                                'NMMODELOAUDIT' => $rel['NMMODELOAUDIT'] ?? 'Modelo não disponível',
                                'NRMODELOAUDIT' => $rel['NRMODELOAUDIT'] ?? 'Código não disponível',
                                'NMCATEGORIAAUDIT' => $rel['NMCATEGORIAAUDIT'] ?? 'Nome não disponível',
                            ];
                        }
                    }
                }
            }

            if (!empty($categoriasExcluir)) {
                $this->ModeloAuditoriaFactory->deleteCategoria($this->FindService->formatarArray($categoriasExcluir));
                $mensagem['excluidos'] = count($categoriasExcluir);
            }

            if (!empty($categoriasRelacionadasMap)) {
                $mensagem['status'] = 'erro';
                $mensagem['nao_excluidos'] = array_values($categoriasRelacionadasMap);
            }

            $this->connection->commit();

            return $mensagem;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }


    // Funções Tab Tópicos
    public function salvarTopicosAuditoria($parametros) 
    {
        $this->connection->beginTransaction();
    
        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();
    
            foreach ($parametros as $topico) {
                if (!isset($topico['NRTOPICOAUDIT']) || $topico['NRTOPICOAUDIT'] == 0) {
                    $param = array(
                        "NRTOPICOAUDIT" => $this->newCode->getCodeByTableName("FCM_TOPICOAUDIT", $NRORG, 1, 10, true),
                        "NMTOPICOAUDIT" => $topico['NMTOPICOAUDIT'],
                        "NOTATOPICAUDIT" => isset($topico['NOTATOPICAUDIT']) ? $topico['NOTATOPICAUDIT'] : 0,
                        "NRORG" => $NRORG,
                        "IDATIVO" => $topico['IDATIVO'],
                        "CDOPERINCLUSAO" => $cdOperador,
                        "NRORGINCLUSAO" => $NRORG
                    );
    
                    $this->ModeloAuditoriaFactory->insertTopicosAuditoria($param);
                } elseif ($topico['IDATIVO'] === 'S') {
                    $param = array(
                        "NRTOPICOAUDIT" => $topico['NRTOPICOAUDIT'],
                        "NMTOPICOAUDIT" => $topico['NMTOPICOAUDIT'],
                        "NOTATOPICAUDIT" => isset($topico['NOTATOPICAUDIT']) ? $topico['NOTATOPICAUDIT'] : 0,
                        "IDATIVO" => $topico['IDATIVO'],
                        "CDOPERULTATU" => $cdOperador,
                        "NRORGULTATU" => $NRORG
                    );
    
                    $this->ModeloAuditoriaFactory->editTopicoAuditoria($param);
                } else {
                    $topicosStr = implode(',', array_map('intval', [$topico['NRTOPICOAUDIT']]));
                    $relacaoTopicos = $this->ModeloAuditoriaFactory->getRelacaoTopicosInatividade($topicosStr);
    
                    if (is_array($relacaoTopicos) && !empty($relacaoTopicos)) {
                        return [
                            'status' => 'erro',
                            'mensagem' => 'Não é possível excluir. Existem tópicos relacionados a outros elementos.',
                            'relacionados' => $relacaoTopicos,
                        ];
                    } elseif (is_int($relacaoTopicos) && $relacaoTopicos > 0) {
                        return $relacaoTopicos;
                    } else {
                        $param = array(
                            "NRTOPICOAUDIT" => $topico['NRTOPICOAUDIT'],
                            "NMTOPICOAUDIT" => $topico['NMTOPICOAUDIT'],
                            "NOTATOPICAUDIT" => isset($topico['NOTATOPICAUDIT']) ? $topico['NOTATOPICAUDIT'] : 0,
                            "IDATIVO" => $topico['IDATIVO'],
                            "CDOPERULTATU" => $cdOperador,
                            "NRORGULTATU" => $NRORG
                        );
    
                        $this->ModeloAuditoriaFactory->editTopicoAuditoria($param);
                    }
                }
            }
    
            $this->connection->commit();
            return ['status' => 'sucesso'];
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }    

    public function getTopicosAuditoria()
    {
        return $this->ModeloAuditoriaFactory->getTopicosAuditoria();
    }


        public function excluirTopicos($parametros)
        {
            $this->connection->beginTransaction();

            try {
                if (!isset($parametros['TOPICOS']) || empty($parametros['TOPICOS'])) {
                    return [
                        'status' => 'erro',
                        'mensagem' => 'O parâmetro TOPICOS não foi fornecido ou está vazio.',
                    ];
                }

                $mensagem = ['status' => 'ok', 'excluidos' => 0, 'nao_excluidos' => []];
                $topicos = $this->ModeloAuditoriaFactory->getRelacaoTopicos($this->FindService->formatarArray($parametros['TOPICOS']));
                $topicosExcluir = [];
                $topicosRelacionados = [];

                $modelosRelacionados = [];

                foreach ($parametros['TOPICOS'] as $topico) {
                    $dataParam = ["NRTOPICOAUDIT" => $topico];
                    $relacionamentos = $this->ModeloAuditoriaFactory->verificaRelacionamentoTopicos($dataParam);

                    if (empty($relacionamentos)) {
                        $topicosExcluir[] = intval($topico);
                    } else {
                        foreach ($relacionamentos as $rel) {
                            $nomeModelo = $rel['NMMODELOAUDIT'] ?? 'Modelo não disponível';

                            if (!isset($modelosRelacionados[$nomeModelo])) {
                                $modelosRelacionados[$nomeModelo] = ['NMMODELOAUDIT' => $nomeModelo];
                            }
                        }
                    }
                }

                if (!empty($topicosExcluir)) {
                    $this->ModeloAuditoriaFactory->deleteTopicos($this->FindService->formatarArray($topicosExcluir));
                    $mensagem['excluidos'] = count($topicosExcluir);
                }

                if (!empty($modelosRelacionados)) {
                    $mensagem['status'] = 'erro';
                    $mensagem['nao_excluidos'] = array_values($modelosRelacionados);
                }

                $this->connection->commit();
                return $mensagem;
            } catch (\Exception $e) {
                $this->connection->rollback();
                throw $e;
            }
        }

    // Funções da TAB de Critérios

    public function salvarCriteriosAvaliacao($parametros)
    {
        $this->connection->beginTransaction();
    
        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();
    
            foreach ($parametros as $criterio) {
                if (!isset($criterio['NRCRITERIOAUDIT']) || $criterio['NRCRITERIOAUDIT'] == 0) {
                    $param = array(
                        "NRCRITERIOAUDIT" => $this->newCode->getCodeByTableName("FCM_CRITERIOAUDIT", $NRORG, 1, 10, true),
                        "NMCRITERIOAUDIT" => $criterio['NMCRITERIOAUDIT'],
                        "IDSOMANOTA" => isset($criterio['IDSOMANOTA']) ? $criterio['IDSOMANOTA'] : 'N',
                        "IDSOMAPERCENTUAL" => isset($criterio['IDSOMAPERCENTUAL']) ? $criterio['IDSOMAPERCENTUAL'] : 'N',
                        "NRORG" => $NRORG,
                        "IDATIVO" => $criterio['IDATIVO'],
                        "CDOPERINCLUSAO" => $cdOperador,
                        "NRORGINCLUSAO" => $NRORG
                    );
    
                    $this->ModeloAuditoriaFactory->insertCriteriosAvaliacao($param);
                } elseif ($criterio['IDATIVO'] === 'S') {
                    $param = array(
                        "NRCRITERIOAUDIT" => $criterio['NRCRITERIOAUDIT'],
                        "NMCRITERIOAUDIT" => $criterio['NMCRITERIOAUDIT'],
                        "IDSOMANOTA" => isset($criterio['IDSOMANOTA']) ? $criterio['IDSOMANOTA'] : 'N',
                        "IDSOMAPERCENTUAL" => isset($criterio['IDSOMAPERCENTUAL']) ? $criterio['IDSOMAPERCENTUAL'] : 'N',
                        "IDATIVO" => $criterio['IDATIVO'],
                        "CDOPERULTATU" => $cdOperador,
                        "NRORGULTATU" => $NRORG
                    );
    
                    $this->ModeloAuditoriaFactory->editCriteriosAvaliacao($param);
                } else {
                    $paramsCriterio = is_array($criterio['NRCRITERIOAUDIT']) 
                        ? implode(',', array_map('intval', $criterio['NRCRITERIOAUDIT'])) 
                        : $criterio['NRCRITERIOAUDIT'];
    
                    $relacaoCriterios = $this->ModeloAuditoriaFactory->getRelacaoCriteriosAtualiza($paramsCriterio);
    
                    if (is_array($relacaoCriterios) && !empty($relacaoCriterios)) {
                        return [
                            'status' => 'erro',
                            'mensagem' => 'Não é possível excluir. Existem critérios relacionados a outros elementos.',
                            'relacionados' => $relacaoCriterios,
                        ];
                    } elseif (is_int($relacaoCriterios) && $relacaoCriterios > 0) {
                        return $relacaoCriterios;
                    } else {
                        $param = array(
                            "NRCRITERIOAUDIT" => $criterio['NRCRITERIOAUDIT'],
                            "NMCRITERIOAUDIT" => $criterio['NMCRITERIOAUDIT'],
                            "IDATIVO" => $criterio['IDATIVO'],
                            "IDSOMANOTA" => $criterio['IDSOMANOTA'],
                            "IDSOMAPERCENTUAL" => $criterio['IDSOMAPERCENTUAL'],
                            "CDOPERULTATU" => $cdOperador,
                            "NRORGULTATU" => $NRORG
                        );
    
                        $this->ModeloAuditoriaFactory->editCriteriosAvaliacao($param);
                    }
                }
            }
    
            $this->connection->commit();
            return ['status' => 'sucesso'];
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
        public function getCriteriosAvaliacao()
    {
        $parametrizacao = $this->getParametrizacao();
        $criterios = $this->ModeloAuditoriaFactory->getCriteriosAvaliacao();

        if ($parametrizacao[0]['IDUTILIZANOTAAUDIT'] === 'N') {
            foreach ($criterios['data'] as &$criterio) {
                $criterio['IDSOMANOTA'] = 'N';
            }
        }

        if ($parametrizacao[0]['IDUTILIZAPERCENTUALAUDIT'] === 'N') {
            foreach ($criterios['data'] as &$criterio) {
                $criterio['IDSOMAPERCENTUAL'] = 'N';
            }
        }
        return $criterios;
    }

    public function excluirCriterios($parametros)
    {
        $this->connection->beginTransaction();

        try {
            if (!isset($parametros['CRITERIOS']) || empty($parametros['CRITERIOS'])) {
                return [
                    'status' => 'erro',
                    'mensagem' => 'O parâmetro CRITERIOS não foi fornecido ou está vazio.',
                ];
            }

            $mensagem = ['status' => 'ok', 'excluidos' => 0, 'nao_excluidos' => []];

            $criteriosRelacionados = $this->ModeloAuditoriaFactory->getRelacaoCriterios(
                $this->FindService->formatarArray($parametros['CRITERIOS'])
            );

            $criteriosExcluir = [];
            $criteriosRelacionadosDetalhados = [];

            $criteriosMap = [];

            foreach ($parametros['CRITERIOS'] as $criterio) {
                $dataParam = ["NRCRITERIOAUDIT" => $criterio];
                $relacionamentos = $this->ModeloAuditoriaFactory->verificaRelacionamentoCriterios($dataParam);

                if (empty($relacionamentos)) {
                    $criteriosExcluir[] = intval($criterio);
                } else {
                    foreach ($relacionamentos as $rel) {
                        $key = $rel['NRMODELOAUDIT'] . '-' . $rel['NMMODELOAUDIT'];
                        if (!isset($criteriosMap[$key])) {
                            $criteriosMap[$key] = [
                                'NMMODELOAUDIT' => $rel['NMMODELOAUDIT'] ?? 'Modelo não disponível',
                                'NRMODELOAUDIT' => $rel['NRMODELOAUDIT'] ?? 'Modelo não disponível',
                                'NRCRITERIOAUDIT' => $rel['NRCRITERIOAUDIT'] ?? 'Critério não disponível',
                            ];
                        }
                    }
                }
            }

            if (!empty($criteriosExcluir)) {
                $this->ModeloAuditoriaFactory->deleteCriterios($this->FindService->formatarArray($criteriosExcluir));
                $mensagem['excluidos'] = count($criteriosExcluir);
            }

            if (!empty($criteriosMap)) {
                $mensagem['status'] = 'erro';
                $mensagem['nao_excluidos'] = array_values($criteriosMap);
            }

            $this->connection->commit();
            return $mensagem;
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    // Funções gerais
    public function excluirRegistros($parametros)
    {
        if (isset($parametros['CRITERIOS'])) {
            return $this->excluirCriterios($parametros);
        } elseif (isset($parametros['TOPICOS'])) {
            return $this->excluirTopicos($parametros);
        } elseif (isset($parametros['CATEGORIAS'])) {
            return $this->excluirCategoria($parametros);
        } elseif (isset($parametros['AUDITORIA'])) {
            return $this->excluirAuditoria($parametros);
        } elseif (isset($parametros['DADOS'])) {
            return $this->excluirDados($parametros);
        } elseif (isset($parametros['MODELO'])) {
            return $this->excluirModelo($parametros);
        }
    }

    public function findLocalAuditoriaCli()
    {
        return $this->ModeloAuditoriaFactory->findLocalAuditoriaCli();
    }

    /* Funções da tela de execução de uma auditoria */

    public function getCategoriasAuditoria($parametros)
    {
        try {
            $categorias = $this->ModeloAuditoriaFactory->getCategoriasAuditoria(array(
                "NRAUDITORIA" => $parametros['NRAUDITORIA']
            ));

            $categoriasPercentual = $this->getPercentualCategoria($categorias, $parametros);

            return $this->getNotaCategoria($categoriasPercentual, $parametros);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getTopicosAuditoriaExec($parametros)
    {
        return $this->ModeloAuditoriaFactory->getTopicosAuditoriaExec(array(
            "NRAUDITORIA" => $parametros['NRAUDITORIA'],
            "NRCATEGORIAAUDIT" => $parametros['NRCATEGORIAAUDIT'],
        ));
    }


    public function getCriteriosAuditoriaExec($parametros)
    {
        return $this->ModeloAuditoriaFactory->getCriteriosAuditoriaExec(array(
            "NRCATEGORIAAUDIT" => $parametros['NRCATEGORIAAUDIT'],
            "NRAUDITORIA" => $parametros['NRAUDITORIA'],
            "NRTOPICOAUDIT" => $parametros['NRTOPICOAUDIT'],
        ));
    }

    public function alterarStatusCriterio($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $cdOperador = $this->Environment->getCdOperador();
            $nrorg = $this->Environment->getNrOrg();

            if (!empty($parametros['criteriosN'])) {
                $this->ModeloAuditoriaFactory->updateCriteriosResultado(array(
                    "CDOPERULTATU" => $cdOperador,
                    "NRORGULTATU" => $nrorg,
                    "IDREALIZADO" => 'N',
                ), $this->FindService->formatarArray($parametros['criteriosN']));
            }

            if (!empty($parametros['criteriosS'])) {

                $this->ModeloAuditoriaFactory->updateCriteriosResultado(array(
                    "CDOPERULTATU" => $cdOperador,
                    "NRORGULTATU" => $nrorg,
                    "IDREALIZADO" => 'S',
                ), $this->FindService->formatarArray($parametros['criteriosS']));
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function iniciarOuFinalizarAuditoria($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $cdOperador = $this->Environment->getCdOperador();
            $nrorg = $this->Environment->getNrOrg();

            $this->ModeloAuditoriaFactory->updateAuditoria(array(
                "NRAUDITORIA" => $parametros['NRAUDITORIA'],
                "IDSITUACAOAUDIT" => $parametros['IDSITUACAOAUDIT'],
                "CDOPERULTATU" => $cdOperador,
                "NRORGULTATU" => $nrorg,
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function salvarDadosObservacao($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            foreach ($parametros as $param) {
                // verificar existência de observação para a respectiva categoria
                $existeObservacao = $this->ModeloAuditoriaFactory->existenciaObservacao(array(
                    "NRAUDITORIA" => $param["NRAUDITORIA"],
                    "NRRELACCATTOPIC" => $param['NRRELACCATTOPIC'],
                ));

                //Se não existir uma observação registrada na categoria selecionada, será armazenado um novo registro de observação.
                if (count($existeObservacao) > 0) {
                    // Realiza edição do campo Observação para a respectiva categoria
                    $this->ModeloAuditoriaFactory->updateObservacaoAuditoria(array(
                        "NROBSRESULTAUDIT" => $existeObservacao[0]['NROBSRESULTAUDIT'],
                        "NRAUDITORIA" => $param["NRAUDITORIA"],
                        "NRRELACCATTOPIC" => $param['NRRELACCATTOPIC'],
                        "DSOBSERVACAO" => $param['DSOBSERVACAO'],
                        "NRORG" => $NRORG,
                        "CDOPERULTATU" => $cdOperador,
                        "NRORGULTATU" => $NRORG
                    ));
                } else {
                    $this->ModeloAuditoriaFactory->insertObservacaoAuditoria(array(
                        "NROBSRESULTAUDIT" => $this->newCode->getCodeByTableName("FCM_OBSRESULTAUDIT", $NRORG, 1, 10, true),
                        "NRAUDITORIA" => $param["NRAUDITORIA"],
                        "NRRELACCATTOPIC" => $param['NRRELACCATTOPIC'],
                        "DSOBSERVACAO" => $param['DSOBSERVACAO'],
                        "NRORG" => $NRORG,
                        "CDOPERINCLUSAO" => $cdOperador,
                        "NRORGINCLUSAO" => $NRORG
                    ));
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function anexarImagemAuditoria($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            $imagens = $this->FileServerService->uploadFiles($parametros['IMAGENS'], 'anexosAuditoria');

            foreach ($imagens as $img) {
                $this->ModeloAuditoriaFactory->anexarImagemAuditoria(array(
                    "NRFOTORESULTAUDIT" => $this->newCode->getCodeByTableName("FCM_FOTORESULTAUDIT", $NRORG, 1, 10, true),
                    "NRAUDITORIA" => $parametros['NRAUDITORIA'],
                    "NRCATEGORIAAUDIT" => $parametros['NRCATEGORIAAUDIT'],
                    "NRTOPICOAUDIT" => $parametros['NRTOPICOAUDIT'],
                    "DSFOTOAUDITORIA" => $this->fileServerUrl . '/files/' . $img['path'],
                    "NRORG" => $NRORG,
                    "CDOPERINCLUSAO" => $cdOperador,
                    "NRORGINCLUSAO" => $NRORG,
                    "DSTYPE" => $img['type'],
                    "NRSIZE" => $img['size']
                ));
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getImagensTopicosAuditoria($parametros)
    {
        return $this->ModeloAuditoriaFactory->getImagensTopicosAuditoria(array(
            "NRAUDITORIA" => $parametros['NRAUDITORIA'],
            "NRCATEGORIAAUDIT" => $parametros['NRCATEGORIAAUDIT'],
            "NRTOPICOAUDIT" => $parametros['NRTOPICOAUDIT'],
        ));
    }

    public function excluirImagemAuditoria($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $this->ModeloAuditoriaFactory->excluirImagemAuditoria(array(
                "NRFOTORESULTAUDIT" => $parametros['NRFOTORESULTAUDIT'],
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function verificarAlteracoes($parametros)
    {
        try {
            foreach ($parametros['ALTERACOES'] as $value) {
                // Verifica se existe no backend
                $haAlteracao = $this->ModeloAuditoriaFactory->verificarAlteracoes([
                    "NRAUDITORIA" => $value["NRAUDITORIA"],
                    "NRRELACCATTOPIC" => $value['NRRELACCATTOPIC'],
                    "DSOBSERVACAO" => $value['DSOBSERVACAO'],
                ]);

                // Se a contagem for zero, a observação não existe no backend
                if ($haAlteracao[0]['CONT'] == 0) {
                    return true; // Não há nada no backend
                }
            }
            // Se passou por todas as observações e não encontrou nenhuma alteração
            return false;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function calcularNota($params)
    {

        try {
            $parametrizacao = $this->getParametrizacao();

            if ($parametrizacao[0]['IDUTILIZANOTAAUDIT'] === 'S') {
                // Pega todas as categorias
                $CriteriosSomaNota = $this->ModeloAuditoriaFactory->getCriteriosAvaliacaoSomaNota();
                $categorias = $this->getCategoriasAuditoria($params);
                $resultados = [];
                $criteriosValidos = array_column($CriteriosSomaNota, 'NRCRITERIOAUDIT');


                foreach ($categorias['data'] as $categoria) {
                    $topicos = $this->getTopicosAuditoriaExec(array(
                        'NRAUDITORIA' => $params['NRAUDITORIA'],
                        'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'],
                    ));

                    foreach ($topicos['data'] as $topico) {
                        $criterios = $this->ModeloAuditoriaFactory->getCriteriosMarcadosAudit(array(
                            'NRTOPICOAUDIT' => $topico['NRTOPICOAUDIT'],
                            'NRAUDITORIA' => $params['NRAUDITORIA'],
                            'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'],
                        ));


                        foreach ($criterios as $criterio) {
                            if (in_array($criterio['NRCRITERIOAUDIT'], $criteriosValidos)) {
                                $resultados[] = array(
                                    'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'],
                                    'NRTOPICOAUDIT' => $topico['NRTOPICOAUDIT'],
                                    'NRCRITERIOAUDIT' => $criterio['NRCRITERIOAUDIT'],
                                );
                            }
                        }
                    }
                }

                $notaTotal = 0;
                // Calcula a nota total dos tópicos que passaram na validação
                foreach ($resultados as $resultado) {
                    $notaTopico = $this->ModeloAuditoriaFactory->getNotaTopico($resultado);

                    if (!empty($notaTopico)) {
                        $notaTotal += $notaTopico[0]['NOTATOPICAUDIT'];
                        // Soma as notas do tópico
                    }
                }

                $this->ModeloAuditoriaFactory->salvarNota(array(
                    'NOTATOTAL' => $notaTotal,
                    'NRAUDITORIA' => $params['NRAUDITORIA'],
                    'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'],
                ));
            }

            return round($notaTotal, 2);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function calcularPercentual($params)
    {

        try {
            $parametrizacao = $this->getParametrizacao();

            if ($parametrizacao[0]["IDUTILIZAPERCENTUALAUDIT"] === 'S') {
                $CriteriosSomaPercentual = $this->ModeloAuditoriaFactory->getCriteriosAvaliacaoSomaPercentual();
                $categorias = $this->getCategoriasAuditoria($params);

                $criteriosValidos = array_column($CriteriosSomaPercentual, 'NRCRITERIOAUDIT');

                $criteriosSomaPercentualTotal = 0;
                $criteriosSomaPercentualMarcados = 0;

                foreach ($categorias['data'] as $categoria) {
                    $topicos = $this->getTopicosAuditoriaExec(array(
                        'NRAUDITORIA' => $params['NRAUDITORIA'],
                        'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'],
                    ));
                    foreach ($topicos['data'] as $topico) {
                        $criterios = $this->ModeloAuditoriaFactory->getCriteriosAudit(array(
                            'NRTOPICOAUDIT' => $topico['NRTOPICOAUDIT'],
                            'NRAUDITORIA' => $params['NRAUDITORIA'],
                        ));
                        foreach ($criterios as $criterio) {
                            if (in_array($criterio['NRCRITERIOAUDIT'], $criteriosValidos)) {

                                $criteriosSomaPercentualTotal++;
                                if ($criterio['IDREALIZADO'] == 'S') {
                                    $criteriosSomaPercentualMarcados++;
                                }
                            }
                        }
                    }
                }

                if ($criteriosSomaPercentualTotal == 0) {
                    $this->ModeloAuditoriaFactory->salvarPercentual(array(
                        'PERCENTUALCONCLUSAO' => 0,
                        'NRAUDITORIA' => $params['NRAUDITORIA'],
                    ));
                } else {
                    $percentualConclusao = ($criteriosSomaPercentualMarcados / $criteriosSomaPercentualTotal) * 100;
                    $percentualConclusao = round($percentualConclusao, 0);
                    $this->ModeloAuditoriaFactory->salvarPercentual(array(
                        'PERCENTUALCONCLUSAO' => $percentualConclusao,
                        'NRAUDITORIA' => $params['NRAUDITORIA'],
                    ));
                }
            }

            return $percentualConclusao;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getParametrizacao()
    {
        try {
            return $this->ModeloAuditoriaFactory->getParametrizacao();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getNotaCategoria($categorias, $params)
    {
        try {
            $parametrizacao = $this->getParametrizacao();

            if ($parametrizacao[0]['IDUTILIZANOTAAUDIT'] === 'S') {
                $CriteriosSomaNota = $this->ModeloAuditoriaFactory->getCriteriosAvaliacaoSomaNota();
                $criteriosValidos = array_column($CriteriosSomaNota, 'NRCRITERIOAUDIT');

                foreach ($categorias['data'] as &$categoria) {
                    $resultados = [];

                    $topicos = $this->getTopicosAuditoriaExec(array(
                        'NRAUDITORIA' => $params['NRAUDITORIA'] ?? null,
                        'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'] ?? null,
                    ));


                    if (!is_array($topicos) || !isset($topicos['data']) || !is_array($topicos['data'])) {
                        $topicos = ['data' => []];
                    }

                    foreach ($topicos['data'] as $topico) {
                        $arryCrit = array(
                            'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'] ?? null,
                            'NRTOPICOAUDIT' => $topico['NRTOPICOAUDIT'] ?? null,
                            'NRAUDITORIA' => $params['NRAUDITORIA'] ?? null,
                        );

                        $criterios = $this->ModeloAuditoriaFactory->getCriteriosMarcadosAudit($arryCrit);

                        foreach ($criterios as $criterio) {
                            if (in_array($criterio['NRCRITERIOAUDIT'], $criteriosValidos)) {
                                $resultados[] = array(
                                    'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'],
                                    'NRTOPICOAUDIT' => $topico['NRTOPICOAUDIT'],
                                    'NRCRITERIOAUDIT' => $criterio['NRCRITERIOAUDIT'],
                                );
                            }
                        }
                    }

                    $notaTotalCategoria = 0;

                    foreach ($resultados as $resultado) {
                        $notaTopico = $this->ModeloAuditoriaFactory->getNotaTopico($resultado);

                        if (!empty($notaTopico)) {
                            $notaTotalCategoria += $notaTopico[0]['NOTATOPICAUDIT'];
                        }
                    }
                    $categoria['NOTA'] = round($notaTotalCategoria, 2);
                }

                return $categorias;
            } else {
                foreach ($categorias['data'] as &$categoria) {
                    $categoria['NOTA'] = 'N/A';
                }

                return $categorias;
            }
        } catch (\Exception $e) {
            error_log("Erro em getNotaCategoria: " . $e->getMessage());
            throw $e;
        }
    }

    public function getPercentualCategoria($categorias, $params)
    {
        try {
            $parametrizacao = $this->getParametrizacao();

            if ($parametrizacao[0]["IDUTILIZAPERCENTUALAUDIT"] !== 'S') {
                foreach ($categorias['data'] as &$categoria) {
                    $categoria['PERCENTUAL'] = 'N/A';
                }

                return $categorias;
            }

            $CriteriosSomaPercentual = $this->ModeloAuditoriaFactory->getCriteriosAvaliacaoSomaPercentual();
            $criteriosValidos = array_column($CriteriosSomaPercentual, 'NRCRITERIOAUDIT');
            $percentuaisPorCategoria = [];

            foreach ($categorias['data'] as &$categoria) {
                $topicos = $this->getTopicosAuditoriaExec(array(
                    'NRAUDITORIA' => $params['NRAUDITORIA'] ?? null,
                    'NRCATEGORIAAUDIT' => $categoria['NRCATEGORIAAUDIT'] ?? null,
                ));

                if (!is_array($topicos) || !isset($topicos['data']) || !is_array($topicos['data'])) {
                    $topicos = ['data' => []];
                }

                $criteriosSomaPercentualTotal = 0;
                $criteriosSomaPercentualMarcados = 0;

                foreach ($topicos['data'] as $topico) {
                    $criterios = $this->ModeloAuditoriaFactory->getCriteriosAudit(array(
                        'NRTOPICOAUDIT' => $topico['NRTOPICOAUDIT'] ?? null,
                        'NRAUDITORIA' => $params['NRAUDITORIA'] ?? null,
                    ));

                    foreach ($criterios as $criterio) {
                        if (in_array($criterio['NRCRITERIOAUDIT'], $criteriosValidos)) {
                            $criteriosSomaPercentualTotal++;
                            if ($criterio['IDREALIZADO'] == 'S') {
                                $criteriosSomaPercentualMarcados++;
                            }
                        }
                    }
                }

                if ($criteriosSomaPercentualTotal > 0) {
                    $percentualConclusao = ($criteriosSomaPercentualMarcados / $criteriosSomaPercentualTotal) * 100;
                    $percentuaisPorCategoria = round($percentualConclusao, 0);
                } else {
                    $percentuaisPorCategoria = 0;
                }

                $categoria['PERCENTUAL'] = $percentuaisPorCategoria;
            }

            return $categorias;
        } catch (\Exception $e) {
            error_log("Erro em getPercentualCategoria: " . $e->getMessage());
            throw $e;
        }
    }

    public function getNotaPorCategoria($params)
    {
        try {
            $CriteriosSomaNota = $this->ModeloAuditoriaFactory->getCriteriosAvaliacaoSomaNota();

            $criteriosValidos = array_column($CriteriosSomaNota, 'NRCRITERIOAUDIT');

            $topicos = $this->getTopicosAuditoriaExec(array(
                'NRAUDITORIA' => $params['NRAUDITORIA'],
                'NRCATEGORIAAUDIT' => $params['NRCATEGORIAAUDIT'],
            ));

            foreach ($topicos['data'] as $topico) {
                $criterios = $this->ModeloAuditoriaFactory->getCriteriosMarcadosAudit(array(
                    'NRTOPICOAUDIT' => $topico['NRTOPICOAUDIT'],
                    'NRAUDITORIA' => $params['NRAUDITORIA'],
                ));


                foreach ($criterios as $criterio) {
                    if (in_array($criterio['NRCRITERIOAUDIT'], $criteriosValidos)) {
                        $resultados[] = array(
                            'NRCATEGORIAAUDIT' => $params['NRCATEGORIAAUDIT'],
                            'NRTOPICOAUDIT' => $topico['NRTOPICOAUDIT'],
                            'NRCRITERIOAUDIT' => $criterio['NRCRITERIOAUDIT'],
                        );
                    }
                }
            }
            $notaTotalCategoria = 0;

            foreach ($resultados as $resultado) {
                $notaTopico = $this->ModeloAuditoriaFactory->getNotaTopico($resultado);

                if (!empty($notaTopico)) {
                    $notaTotalCategoria += $notaTopico[0]['NOTATOPICAUDIT'];
                }
            }

            return round($notaTotalCategoria, 2);
        } catch (\Exception $e) {
            throw $e;
        }
    }

     /* Rotina Kanban Auditorias */

     public function getTarefasAuditoriaKanban($parametros)
     {
        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);
        
        if (!empty($parametros['IDSITUACAOAUDIT'])) {
            $params = array(
                "CDOPERADOR"        => $cdoperador,
                "IDSITUACAOAUDIT"   => $parametros['IDSITUACAOAUDIT'],
            );
        }

        return $this->ModeloAuditoriaFactory->getTarefasAuditoriaKanban($params, $listaOperadores);
     }
 
     /* Fim rotina Kanban Auditorias */
}
