<?php

namespace FM\Service;

use FM\Factory\FmtOrdemServFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FileServerService;

class FmtOrdemServService
{
    protected $connection;
    protected $OrdemServFactory;
    protected $Environment;
    protected $newCode;
    protected $FileServerService;
    protected $fileServerUrl;

    public function __construct(EntityManager $entityManager, FmtOrdemServFactory $OrdemServFactory, Environment $Environment, NewCode $newCode, FileServerService $FileServerService, $fileServerUrl)
    {
        $this->OrdemServFactory = $OrdemServFactory;
        $this->connection = $entityManager->getConnection();
        $this->Environment = $Environment;
        $this->newCode = $newCode;
        $this->FileServerService = $FileServerService;
        $this->fileServerUrl = $fileServerUrl;
    }

    public function getOrdensServico()
    {
        return $this->OrdemServFactory->getOrdensServico();
    }

    public function getComentariosOrdem($parametros)
    {
        $parametroComentario = array(
            "NRORDEMSERV" => $parametros["NRORDEMSERV"]
        );

        return $this->OrdemServFactory->getComentariosOrdem($parametroComentario);
    }

    public function saveComentarioOrdem($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $NRCOMENTORDEMSERV = $this->newCode->getCodeByTableName("FCM_COMENTORDEMSERV", $nrorg, 1, 10, true);
            
            $parametroComentario = array(
                "NRCOMENTORDEMSERV" => $NRCOMENTORDEMSERV,
                "NRORDEMSERV" => $parametros["NRORDEMSERV"],
                "DSCOMENTARIO" => $parametros["DSCOMENTARIO"]
            );

            $this->OrdemServFactory->saveComentarioOrdem($parametroComentario);

            $paramHistorico = array(
                "NRORDEMSERV" => $parametros["NRORDEMSERV"],
                "DSORDEMSERVHISTORICO" => "Comentário adicionado à ordem de serviço",
            );

            $this->insertHistorico($paramHistorico);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
        } 
    }

    public function insertOrdemServico($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrorg();
            $NRORDEMSERV = $this->newCode->getCodeByTableName("FCM_ORDEMSERV", $nrorg, 1, 10, true);

            $DSORDEMSERVHISTORICO = "Ordem de serviço criada";

            $parametroOrdem = array(
                "NRORDEMSERV"           => $NRORDEMSERV,
                "NMORDEMOSERV"          => $parametros["NMORDEMOSERV"],
                "NREQUIPAMENTO"         => $parametros["NREQUIPAMENTO"],
                "NRCATEGORIAMANUT"      => $parametros["NRCATEGORIAMANUT"],
                "NRTIPOMANUTENCAO"      => $parametros["NRTIPOMANUTENCAO"],
                "NRPRIORIDADEMANUT"     => $parametros["NRPRIORIDADEMANUT"],
                "DTORDEMSERV"           => $parametros["DTORDEMSERV"],
                "IDSITUACAO"            => $parametros["IDSITUACAO"],
                "DSORDEMSERV"           => $parametros["DSORDEMSERV"],
                "NRSOLICITACAOSERV"     => $parametros["NRSOLICITACAOSERV"] ?? null,
                "NRORG"                 => $nrorg,
                "NRLOCALCLIENTE"        => $parametros['NRLOCALCLIENTE'],
                "NRMODELOPOP"           => $parametros['NRMODELOPOP'],
                "NREQUIPE"              => $parametros['NREQUIPE'],
                "IDTAREFARESPONSAVEL"   => (!empty($parametros['NREQUIPE'])) ? 'N' : 'S'
            );

            $this->OrdemServFactory->insertOrdemServico($parametroOrdem);

            $this->insertRelacaoResponsavelOrdem($parametros, $NRORDEMSERV);

            if ($parametros["SOLICITACAO"]) {
                $NRRELSOLICITAORDEMSERV = $this->newCode->getCodeByTableName("FCM_RELSOLICITAORDEMSERV", $nrorg, 1, 10, true);

                $parametroRelacionamento = array(
                    "NRRELSOLICITAORDEMSERV"    => $NRRELSOLICITAORDEMSERV,
                    "NRORDEMSERV"               => $NRORDEMSERV,
                    "NRSOLICITACAOSERV"         => $parametros["NRSOLICITACAOSERV"]
                );
                $DSORDEMSERVHISTORICO = "Ordem de serviço criada a partir de uma solicitação de serviço";
                $this->OrdemServFactory->insertRelSolicitacaoOrdem($parametroRelacionamento);
            }

            if (count ($parametros["NRESCOPOS"]) > 0) {
                foreach ($parametros["NRESCOPOS"] as $NRESCOPOMANUT) {
                    $NRRELESCORDEMSERV = $this->newCode->getCodeByTableName("FCM_RELESCORDEMSERV", $nrorg, 1, 10, true);
    
                    $parametroEscopo = array(
                        "NRRELESCORDEMSERV" => $NRRELESCORDEMSERV,
                        "NRORDEMSERV"       => $NRORDEMSERV,
                        "NRESCOPOMANUT"     => $NRESCOPOMANUT
                    );
    
                    $this->OrdemServFactory->insertRelEscopoOrdem($parametroEscopo);
                }
            }

            if (count($parametros['NREPI']) > 0) {
                $this->insertRelacaoEpis($parametros, $NRORDEMSERV);
            }
    
            if (isset($parametros["FOTOS"])) {
                $parametroFotos = array(
                    "NRORDEMSERV"   => $NRORDEMSERV,
                    "FOTOS"         => $parametros["FOTOS"]
                );
                $this->insertFotosOrdem($parametroFotos);
            }

            if (isset($parametros["DSCOMENTARIO"])) {
                $this->saveComentarioOrdem(array(
                    "NRORDEMSERV"   => $NRORDEMSERV,
                    "DSCOMENTARIO"  => $parametros["DSCOMENTARIO"]
                ));
            }

            $this->insertHistorico(array(
                "NRORDEMSERV"           => $NRORDEMSERV,
                "NRSOLICITACAOSERV"     => $parametros["NRSOLICITACAOSERV"] ?? null,
                "DSORDEMSERVHISTORICO"  => $DSORDEMSERVHISTORICO,
            ));

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
        }
    }

    public function insertRelacaoEpis($parametros, $NRORDEMSERV)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operadorLogado = $this->Environment->getCdOperador();
        
            foreach ($parametros["NREPI"] as $EPI) {
                $NRRELACEPIORDEMS = $this->newCode->getCodeByTableName("FCM_RELACEPIORDEMS", $nrorg, 1, 10, true);

                $this->OrdemServFactory->insertRelacaoEpis(array(
                    "NRRELACEPIORDEMS"  => $NRRELACEPIORDEMS,
                    "NRORDEMSERV"       => $NRORDEMSERV,
                    "NREPI"             => $EPI,
                    "CDOPERINCLUSAO"    => $operadorLogado,
                    "NRORGINCLUSAO"     => $nrorg,
                ));
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
        }
    }

    public function insertRelacaoResponsavelOrdem($parametros, $NRORDEMSERV)
    {
        $arrayResponsavel = [];
        $responsavel = '';
        $nrorg = $this->Environment->getNrorg();
        $operadorLogado = $this->Environment->getCdOperador();
        $profissionalExterno = array();

        if ($parametros['NRPROFISSIONALEXTERNO'] !== null) {
            array_push($profissionalExterno, $parametros['NRPROFISSIONALEXTERNO']);
        }

        if (!empty($parametros['NOVOPROFISSIONAL'])) {
            $nrprofissionalExterno = $this->insertProfissionalExterno($parametros['NOVOPROFISSIONAL']);
            array_push($profissionalExterno, $nrprofissionalExterno);
        }

        if (!empty($parametros['CDOPERADOR'])) {
            $arrayResponsavel = $parametros['CDOPERADOR'];
            $responsavel = 'operador';
        } else {
            $arrayResponsavel = $profissionalExterno;
            $responsavel = 'externo';
        }

        $this->connection->beginTransaction();

        try {

            foreach ($arrayResponsavel as $resp) {
                $NRRESPONSAVELORDEMS = $this->newCode->getCodeByTableName("FCM_RESPONSAVELORDEMS", $nrorg, 1, 10, true);
                
                $params = array(
                    "NRRESPONSAVELORDEMS"       => $NRRESPONSAVELORDEMS,
                    "NRORDEMSERV"               => $NRORDEMSERV,
                    "NRPROFISSIONALEXTERNO"     => ($responsavel === 'externo') ? $resp : null,
                    "CDOPERADOR"                => ($responsavel === 'operador') ? $resp : null,
                    "CDOPERINCLUSAO"            => $operadorLogado,
                    "NRORG"                     => $nrorg,
                    "NRORGINCLUSAO"             => $nrorg,
                );

                $this->OrdemServFactory->insertRelacaoResponsavelOrdem($params);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
        }

    }
    public function insertProfissionalExterno($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operadorLogado = $this->Environment->getCdOperador();
        
            $nr = $this->newCode->getCodeByTableName("FCM_PROFISSIONALEXTERNO", $nrorg, 1, 10, true);

            $this->OrdemServFactory->insertProfissionalExterno(array(
                "NRPROFISSIONALEXTERNO"   => $nr,
                "NMPROFISSIONALEXTERNO"   => $parametros['NMPROFISSIONALEXTERNO'],
                "NMEMPRESA"               => $parametros['NMEMPRESA'],
                "INSCRICAO"               => $parametros['INSCRICAO'],
                "CDOPERINCLUSAO"          => $operadorLogado,
                "NRORG"                   => $nrorg,
                "NRORGINCLUSAO"           => $nrorg,
            ));

            $this->connection->commit();        
            return $nr;
        } catch (\Exception $e) {
            $this->connection->rollback();
        }
    }
    
    // public function insertRelacaoResponsaveis($parametros, $NRORDEMSERV) {
    //     $this->connection->beginTransaction();

    //     try {
    //         $nrorg = $this->Environment->getNrorg();
    //         $operadorLogado = $this->Environment->getCdOperador();
        
    //         $NRRESPONSAVELORDEMS = $this->newCode->getCodeByTableName("FCM_RESPONSAVELORDEMS", $nrorg, 1, 10, true);

    //         $this->OrdemServFactory->insertRelEscopoOrdem(array(
    //             "NRRESPONSAVELORDEMS"   => $NRRESPONSAVELORDEMS,
    //             "NRORDEMSERV"           => $NRORDEMSERV,
    //             "CDOPERADOR"            => $,
    //             "CDOPERINCLUSAO"    => $operadorLogado,
    //             "NRORGINCLUSAO"     => $nrorg,
    //         ));
    //         $this->connection->commit();
    //     } catch (\Exception $e) {
    //         $this->connection->rollback();
    //     }
    // }

    public function insertFotosOrdem($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();

            foreach ($parametros["FOTOS"] as $foto) {
                $img = $this->FileServerService->uploadFoto([$foto], "foto_manutencao_fm");

                $NRIMGORDEMSERV = $this->newCode->getCodeByTableName('FCM_IMGORDEMSERV', $nrorg, 1, 10, true);
                $parametroFoto = array(
                    "NRIMGORDEMSERV" => $NRIMGORDEMSERV,
                    "NRORDEMSERV" => $parametros["NRORDEMSERV"],
                    "DSCAMINHO" => $this->fileServerUrl . "/files/" . $img["path"],
                    "NRORG" => $nrorg,
                    "DSTYPE" => $img["type"],
                    "NRSIZE" => $img["size"]
                );

                $this->OrdemServFactory->insertFotoOrdem($parametroFoto);


                $this->insertHistorico(array(
                    "NRORDEMSERV" => $parametros["NRORDEMSERV"],
                "DSORDEMSERVHISTORICO" => "Foto adicionada à ordem de serviço",
                ));
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
        }
    }

    public function insertHistorico($param)
    {
        try {
            $this->connection->beginTransaction();
            $nrorg = $this->Environment->getNrorg();

            $this->OrdemServFactory->insertHistorico(array(
                "NRORDEMSERVHISTORICO"  => $this->newCode->getCodeByTableName("FCM_ORDEMSERVHISTORICO", $nrorg, 1, 10, true),
                "NRSOLICITACAOSERV"     => $param["NRSOLICITACAOSERV"] ?? null,
                "NRORDEMSERV"           => $param["NRORDEMSERV"],
                "DSORDEMSERVHISTORICO"  => $param["DSORDEMSERVHISTORICO"],
                "NRORG"                 => $nrorg,
                "CDOPERINCLUSAO"        => $this->Environment->getCdOperador(),
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
        }
    }


    public function getHistoricoOrdemServ($parametros)
    {
        return $this->OrdemServFactory->getHistoricoOrdemServ($parametros);
    }

    /* Rotina do Kanban */

    public function getDadosKanban($parametros)
    {
        if (!empty($parametros['SITUACAO'])) {

            return $this->OrdemServFactory->getDadosKanban(array(
                "IDSITUACAO" => $parametros["SITUACAO"]
            ));
        }

    }
    
    /* Fim rotina do Kanban */

    public function getProfissionalExterno()
    {
        return $this->OrdemServFactory->getProfissionalExterno();
    }
    
}
