import { Utilities } from '@/util/Utilities';
import {
    CheckboxMultiple, LoadingService, Modal, ModalService, Text,
    Textarea,
} from '@zeedhi/common';
import {
    Http, IDictionary, IEventParam, Metadata, SimpleDatasource,
} from '@zeedhi/core';
import { IterableCarousel } from '@zeedhi/teknisa-components-common';
import { KanbanTask } from '@zeedhi/zd-kanban';

export class FmKanbanTarefas {
    public modal!: Modal;

    public nrChecklistAgend: string = '';

    public visibilidadeColunaAtraso({ component }: any) {
        const situacao = component.name.split('_')[1];
        const atraso = component.name.split('_')[2];

        if (situacao === '1' && atraso !== '  ') {
            component.isVisible = true;
        } else {
            component.isVisible = false;
        }
    }

    public async criarModais() {
        this.modal = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00004_000_tarefas/modal/visualizar_infos_kanban_tarefas' });
    }

    public destruirModais() {
        this.modal.destroy();
    }

    public async abrirModal({ component }: IEventParam<KanbanTask>) {
        /* eslint-disable */
        this.nrChecklistAgend = component.name.split('_')[0];
        const idsituacao = component.name.split('_')[1];
        /* eslint-enable */
        if (idsituacao !== '1') {
            this.modal.show();
            this.montarInformacoesComplementares();
            this.carregarFotos();
        }
    }

    public fecharModal() {
        this.modal.hide();
    }

    public async montarInformacoesComplementares() {
        const nome = Metadata.getInstance<Text>('id-tarefa');
        const checkbox = Metadata.getInstance<CheckboxMultiple>('checkbox-itens');
        checkbox.value = [];

        const observacoes = Metadata.getInstance<Textarea>('observacoes-tarefas');
        const semItens = Metadata.getInstance<Text>('nao-ha-itens');
        let mensagem = '';

        LoadingService.show();
        Http.post('/getItensTarefasKanban', {
            NRCHECKLISTAGEND: this.nrChecklistAgend,
        }).then((response: IDictionary) => {
            const dados = response.data.data;
            nome.text = dados[0].NMCHECKLISTAGEND;

            checkbox.value = dados.filter((elemento: any) => elemento.IDREALIZADO === 'S').map((elemento: any) => elemento.NMTAREFA);
            (checkbox.datasource as SimpleDatasource)!.items = dados.map((elemento: any) => elemento.NMTAREFA);

            if (dados.length > 0) {
                semItens.isVisible = false;
            } else {
                semItens.isVisible = true;
            }

            if (dados[0].DSJUSTCANCELAMENTO !== null && dados[0].DSOBSTAREFAAGEND !== null) {
                mensagem = `Justificativa de cancelamento: ${dados[0].DSJUSTCANCELAMENTO} - Observações da Tarefa: ${dados[0].DSOBSTAREFAAGEND}`;
            } else if (dados[0].DSJUSTCANCELAMENTO !== null) {
                mensagem = `Justificativa de cancelamento: ${dados[0].DSJUSTCANCELAMENTO}`;
            } else if (dados[0].DSOBSTAREFAAGEND !== null) {
                mensagem = `Observações da Tarefa: ${dados[0].DSOBSTAREFAAGEND}`;
            } else {
                mensagem = 'Não há observações';
            }
            observacoes.value = mensagem;
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public carregarFotos() {
        const carousel = Metadata.getInstance<IterableCarousel>('carousel-imagens-anexo-vistorias');
        carousel.iterable.datasource.data = [];

        LoadingService.show();
        Http.post('/getFotosTarefas', {
            NRCHECKLISTAGEND: this.nrChecklistAgend,
        }).then((response: any) => {
            if (response.data.data.length === 0) {
                carousel.isVisible = false;
                Metadata.getInstance<Text>('sem-resultado-imagens-tarefas').isVisible = true;
            } else {
                response.data.data.forEach((dados: IDictionary) => {
                    carousel.iterable.datasource.data.push({
                        NRFOTOTAREFAAGEND: dados.NRFOTOTAREFAAGEND,
                        DSFOTOTAREFAAGEND: dados.DSFOTOTAREFAAGEND,
                    });
                });
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }
}
